// Copyright (c) .NET Foundation. All rights reserved.
// Licensed under the Apache License, Version 2.0. See License.txt in the project root for license information.

using System.Collections.Generic;
using JetBrains.Annotations;
using Microsoft.EntityFrameworkCore.Utilities;

namespace Microsoft.EntityFrameworkCore.Metadata.Internal
{
    /// <summary>
    ///     This is an internal API that supports the Entity Framework Core infrastructure and not subject to
    ///     the same compatibility standards as public APIs. It may be changed or removed without notice in
    ///     any release. You should only use it directly in your code with extreme caution and knowing that
    ///     doing so can result in application failures when updating to a new Entity Framework Core release.
    /// </summary>
    public static class ScaffoldingModelExtensions
    {
        /// <summary>
        ///     This is an internal API that supports the Entity Framework Core infrastructure and not subject to
        ///     the same compatibility standards as public APIs. It may be changed or removed without notice in
        ///     any release. You should only use it directly in your code with extreme caution and knowing that
        ///     doing so can result in application failures when updating to a new Entity Framework Core release.
        /// </summary>
        public static IDictionary<string, string> GetEntityTypeErrors([NotNull] this IModel model)
        {
            var errors = (IDictionary<string, string>)model[ScaffoldingAnnotationNames.EntityTypeErrors];
            if (errors == null)
            {
                errors = new Dictionary<string, string>();
                (model as IMutableModel)?.SetEntityTypeErrors(errors);
            }

            return errors;
        }

        /// <summary>
        ///     This is an internal API that supports the Entity Framework Core infrastructure and not subject to
        ///     the same compatibility standards as public APIs. It may be changed or removed without notice in
        ///     any release. You should only use it directly in your code with extreme caution and knowing that
        ///     doing so can result in application failures when updating to a new Entity Framework Core release.
        /// </summary>
        public static void SetEntityTypeErrors([NotNull] this IMutableModel model, [NotNull] IDictionary<string, string> value) =>
            model.SetAnnotation(
                ScaffoldingAnnotationNames.EntityTypeErrors,
                Check.NotNull(value, nameof(value)));

        /// <summary>
        ///     This is an internal API that supports the Entity Framework Core infrastructure and not subject to
        ///     the same compatibility standards as public APIs. It may be changed or removed without notice in
        ///     any release. You should only use it directly in your code with extreme caution and knowing that
        ///     doing so can result in application failures when updating to a new Entity Framework Core release.
        /// </summary>
        public static string GetDatabaseName([NotNull] this IModel model)
            => (string)model[ScaffoldingAnnotationNames.DatabaseName];

        /// <summary>
        ///     This is an internal API that supports the Entity Framework Core infrastructure and not subject to
        ///     the same compatibility standards as public APIs. It may be changed or removed without notice in
        ///     any release. You should only use it directly in your code with extreme caution and knowing that
        ///     doing so can result in application failures when updating to a new Entity Framework Core release.
        /// </summary>
        public static void SetDatabaseName([NotNull] this IMutableModel model, [CanBeNull] string value) =>
            model.SetAnnotation(
                ScaffoldingAnnotationNames.DatabaseName,
                Check.NullButNotEmpty(value, nameof(value)));
    }
}
